#!/usr/bin/env node

const fs = require('fs');
const path = require('path');

const packageInfo = require(path.resolve('package.json'));
const srcPath = './';
const nodeModulesPath = './node_modules/';
const destPath = nodeModulesPath + packageInfo.name;

function installingAsDependency() {
  // ./main-package/node_modules/dependency/package.json
  // ./main-package/package.json
  const rootPackagePath = path.resolve('..', '..', 'package.json');
  if (fs.existsSync(rootPackagePath)) {
    const rootPackageInfo = require(rootPackagePath);
    return rootPackageInfo.name !== packageInfo.name;
  } else {
    return false;
  }
}

if (installingAsDependency()) {
  console.debug('Self-reference was not installed because this package was installed ' +
    'as dependency');
} else {
  console.info(`Installing self-reference to package ${packageInfo.name} ` +
    `at ${path.resolve(destPath)}`);

  // It doesn't create a symlink from the entire directory to avoid to create a recursive
  // package.json inside node_modules. Only relevant directories and files are linked.
  const destDirs = ['lib', 'resources', 'controllers', 'routes', 'routines', 'config',
    'scripts', 'test', 'tests', 'keys', 'context', 'doc', 'polyfills', 'index.js', 'transpiler',
    'startups', 'layouts', 'templates', 'themes', 'publicModules.js'];

  [nodeModulesPath, destPath].forEach(path => {
    if (!fs.existsSync(path)) {
      fs.mkdirSync(path);
      console.info('Created directory', path);
    }
  });

  destDirs.forEach(relativePath => {
    const srcFullPath = path.resolve(srcPath, relativePath);
    if (fs.existsSync(srcFullPath)) {
      const destFullPath = path.resolve(destPath, relativePath);
      const srcStats = fs.lstatSync(srcFullPath);
      if (srcStats.isDirectory() || srcStats.isFile()) {
        if (!fs.existsSync(destFullPath)) {
          if (srcStats.isDirectory()) {
            const kind = /^win/.test(process.platform) ? 'junction' : 'dir';
            fs.symlinkSync(srcFullPath, destFullPath, kind);
            console.info('Symlink', destFullPath, '->', srcFullPath, 'was created');
          } else {
            // Copy file because Windows requires administrator privileges to create symlinks
            fs.writeFileSync(destFullPath, fs.readFileSync(srcFullPath));
            console.info('File', srcFullPath, 'was copied to', destFullPath);
          }
        }
      }
    }
  });
}
