#!/usr/bin/env node

const path = require('path');
const execFileSync = require('child_process').execFileSync;
const pkgInfo = require(path.resolve('package.json'));

const npmPath = /^win/.test(process.platform) ? 'npm.cmd' : 'npm';

function isRegisteredVersion() {
  const output = execFileSync(npmPath, ['info', `${pkgInfo.name}@${pkgInfo.version}`]);
  return output.toString().trim() !== '';
}

if (!isRegisteredVersion()) {
  console.info(`Version ${pkgInfo.version} not found. Publishing the new version`);
  try {
    execFileSync(npmPath, ['publish', process.cwd()]);
  } catch (e) {
    // npm publish return a non-zero exit code when publishing a new version
    if (!isRegisteredVersion()) {
      console.error('Error executing npm publish:', e);
      throw e;
    }
  }
  console.info(`Version ${pkgInfo.version} published successfully`);

} else {
  console.info(`Version ${pkgInfo.version} already published`);
}
