#!/usr/bin/env node

const path = require('path');
const fs = require('fs');
const pacote = require('pacote');

const versionRange = process.argv[2] || '';
if (versionRange || versionRange.match(/^\d*\.\d*$/)) {
  const packageFileName = path.resolve('package.json');
  const packageInfo = require(packageFileName);

  const [current, minimal] = [packageInfo.version, versionRange].map(version => {
    const parts = version.split('.');
    return {
      major: Number(parts[0]),
      minor: Number(parts[1])
    };
  });

  if ((current.major < minimal.major) ||
      (current.major === minimal.major && current.minor < minimal.minor)) {
    const newVersion = minimal.major + '.' + minimal.minor + '.0';
    console.info(`Package version updated from ${packageInfo.version} to ${newVersion}`);
    packageInfo.version = newVersion;
  }

  const erpPackages = (packageInfo.jazDependencies || []).slice(0);
  if ('erp-dev-tools' in packageInfo.dependencies) {
    erpPackages.push('erp-dev-tools');
  }

  Promise.all(erpPackages.map(jazDependency => {
    return pacote.manifest(jazDependency + '@~' + versionRange).then(pkgJson => {
      const latest = '~' + pkgJson.version;
      if (packageInfo.dependencies[jazDependency] !== latest) {
        packageInfo.dependencies[jazDependency] = latest;
        console.info(`Dependency ${jazDependency} updated to ${latest}`);
      }
    }).catch(err => {
      if (err.code === 'ETARGET') {
        const newVersion = '~' + versionRange + '.0';
        console.info(`Dependency ${jazDependency}@ ${versionRange} doesn't exist. Using ${newVersion}`);
        packageInfo.dependencies[jazDependency] = newVersion;
      } else {
        console.error(err);
        throw err;
      }
    });
  })).then(() => {
    if (packageInfo.jazDependencies && packageInfo.jazDependencies.includes('erp-dev-tools')) {
      packageInfo.jazDependencies = packageInfo.jazDependencies.filter(e => e !== 'erp-dev-tools');
    }
    fs.writeFileSync(packageFileName, JSON.stringify(packageInfo, null, 2) + '\n');
    console.info(`File ${packageFileName} updated`);
  }).catch(console.error);

} else {
  console.error('Version range should be informed using the format "major.minor" (ex 18.1)');
  process.exit(1);
}
